@@ -4,6 +4,8 @@ import os
 from random import sample
 
 from nexus import NexusReader, VERSION, NexusFormatException
+from nexus.tools import check_for_valid_NexusReader
+
 
 __author__ = 'Simon Greenhill <simon@simon.net.nz>'
 __doc__ = """treemanip.py - python-nexus tools v%(version)s
@@ -90,9 +92,7 @@ def run_deltree(deltree, nexus_obj, do_print=False):
     :raises AssertionError: if nexus_obj is not a nexus
     :raises NexusFormatException: if nexus_obj does not have a `trees` block
     """
-    assert isinstance(nexus_obj, NexusReader), "Nexus_obj should be a NexusReader instance"
-    if hasattr(nexus_obj, 'trees') == False:
-        raise NexusFormatException("Nexus has no `trees` block")
+    check_for_valid_NexusReader(nexus_obj, required_blocks=['trees'])
         
     new = []
     delitems = parse_deltree(deltree)
@@ -127,9 +127,7 @@ def run_resample(resample, nexus_obj, do_print=False):
     :raises AssertionError: if nexus_obj is not a nexus
     :raises NexusFormatException: if nexus_obj does not have a `trees` block
     """
-    assert isinstance(nexus_obj, NexusReader), "Nexus_obj should be a NexusReader instance"
-    if hasattr(nexus_obj, 'trees') == False:
-        raise NexusFormatException("Nexus has no `trees` block")
+    check_for_valid_NexusReader(nexus_obj, required_blocks=['trees'])
     
     new = []
     try:
@@ -168,9 +166,7 @@ def run_removecomments(nexus_obj, do_print=False):
     :raises AssertionError: if nexus_obj is not a nexus
     :raises NexusFormatException: if nexus_obj does not have a `trees` block
     """
-    assert isinstance(nexus_obj, NexusReader), "Nexus_obj should be a NexusReader instance"
-    if hasattr(nexus_obj, 'trees') == False:
-        raise NexusFormatException("Nexus has no `trees` block")
+    check_for_valid_NexusReader(nexus_obj, required_blocks=['trees'])
     
     new = []
     for index, tree in enumerate(nexus_obj.trees, 1):
@@ -197,13 +193,11 @@ def run_detranslate(nexus_obj, do_print=False):
     :raises AssertionError: if nexus_obj is not a nexus
     :raises NexusFormatException: if nexus_obj does not have a `trees` block
     """
-    assert isinstance(nexus_obj, NexusReader), "Nexus_obj should be a NexusReader instance"
-    if hasattr(nexus_obj, 'trees') == False:
-        raise NexusFormatException("Nexus has no `trees` block")
+    check_for_valid_NexusReader(nexus_obj, required_blocks=['trees'])
     nexus_obj.trees.detranslate()
     return nexus_obj
 
-def run_random(num_trees, nexus_obj):
+def run_random(num_trees, nexus_obj, do_print=False):
     """
     Returns a specified number (`num_trees`) of random trees from the nexus file.
     
@@ -219,18 +213,22 @@ def run_random(num_trees, nexus_obj):
     :raises NexusFormatException: if nexus_obj does not have a `trees` block
     :raises ValueError: if num_trees is not an integer
     :raises ValueError: if num_trees is larger than population
-    
     """
-    assert isinstance(nexus_obj, NexusReader), "Nexus_obj should be a NexusReader instance"
-    if hasattr(nexus_obj, 'trees') == False:
-        raise NexusFormatException("Nexus has no `trees` block")
+    check_for_valid_NexusReader(nexus_obj, required_blocks=['trees'])
     
     try:
         num_trees = int(num_trees)
     except ValueError:
         raise ValueError("num_trees should be an integer")
     
-    nexus_obj.trees.trees = sample(nexus_obj.trees.trees, num_trees)
+    if num_trees > nexus_obj.trees.ntrees:
+        raise ValueError("Treefile only has %d trees in it." % nexus_obj.trees.ntrees)
+    elif num_trees == nexus_obj.trees.ntrees:
+        return nexus_obj # um. ok.
+    else:
+        if do_print:
+            print("%d trees read... sampling %d" % (nexus_obj.trees.ntrees, num_trees))
+        nexus_obj.trees.trees = sample(nexus_obj.trees.trees, num_trees)
     return nexus_obj
     
     
