@@ -71,7 +71,9 @@ class GenericHandler(object):
         
         :return: Returns a cleaned string.
         """
-        return COMMENT_PATTERN.sub('', line)
+        if '[' in line:
+            return COMMENT_PATTERN.sub('', line)
+        return line
         
     def write(self):
         """
@@ -84,7 +86,7 @@ class GenericHandler(object):
 
 class TaxaHandler(GenericHandler):
     """Handler for `taxa` blocks"""
-    is_dimensions = re.compile(r"""dimensions\s*ntax=\s*(\d+)""", re.IGNORECASE)
+    is_dimensions = re.compile(r"""dimensions\s*ntax\s*=\s*(\d+)""", re.IGNORECASE)
     is_taxlabel_block = re.compile(r"""\btaxlabels\b""", re.IGNORECASE)
     
     def __init__(self):
@@ -117,7 +119,6 @@ class TaxaHandler(GenericHandler):
                 in_taxlabel_block = True
             elif in_taxlabel_block:
                 self.taxa.append(line.strip(";"))
-        
         assert self.ntaxa == len(self.taxa)
         
     def write(self):
@@ -140,12 +141,13 @@ class TaxaHandler(GenericHandler):
 
 class TreeHandler(GenericHandler):
     """Handler for `trees` blocks"""
-    is_tree = re.compile(r"""tree .*=.*;""")
+    is_tree = re.compile(r"""tree .*=.*;""", re.IGNORECASE)
     
     def __init__(self):
         self.was_translated = False # does the treefile have a translate block?
         self._been_detranslated = False # has detranslate been called?
         self.translators = {}
+        self.taxa = []
         self.trees = []
         super(TreeHandler, self).__init__()
         
@@ -171,9 +173,7 @@ class TreeHandler(GenericHandler):
         translation_pattern = re.compile(r"""(\d+)\s(['"\w\d\.]+)[,;]?""")
         
         lost_in_translation = False
-        translators = {}
         for line in data:
-            
             # look for translation start, and turn on lost_in_translation
             if translate_start.match(line):
                 lost_in_translation = True
@@ -189,7 +189,7 @@ class TreeHandler(GenericHandler):
                     self.translators[taxon_id] = taxon
                 if line.endswith(';'):
                     lost_in_translation = False
-                    translators = translators
+                    self.taxa = self.translators.values()
                     
             elif self.is_tree.search(line):
                 if lost_in_translation == True:
@@ -197,7 +197,12 @@ class TreeHandler(GenericHandler):
                     raise NexusFormatException("Tree block has incomplete translate table")
                 
                 self.trees.append(line)
-            
+        
+        # get taxa if not translated.
+        if len(self.translators) == 0:
+            self.taxa = re.findall(r"""[(),](\w+)[:),]""", self.trees[0])
+        
+    
     def detranslate(self):
         """Detranslates all trees in the file"""
         if self._been_detranslated == True:
@@ -280,6 +285,7 @@ class DataHandler(GenericHandler):
     def __init__(self):
         self.taxa = []
         self.ntaxa = 0
+        self.symbols = set()
         self.characters = {}
         self.charlabels = {}
         self.nchar = 0
@@ -336,7 +342,9 @@ class DataHandler(GenericHandler):
         ['1', '12']
         >>> DataHandler()._parse_sites('123(4,5)56')
         ['1', '2', '3', '4,5', '5', '6']
-
+        >>> DataHandler()._parse_sites("ACGTU?")
+        ['A', 'C', 'G', 'T', 'U', '?']
+        
         :param sites: string
         :type sites: string
 
@@ -361,11 +369,16 @@ class DataHandler(GenericHandler):
                     else:
                         site += nextchar
             out.append(site)
+            
+            # add to symbol list
+            if site not in self.symbols:
+                self.symbols.update(set(site))
+                
         # check we're not in hanging multistate chunk
         if multistate:
             raise NexusFormatException("Data Matrix contains incomplete multistate values")
         return out
-    
+        
     def parse(self, data):
         """
         Parses a `data` block
@@ -424,7 +437,7 @@ class DataHandler(GenericHandler):
                 
         self._load_characters()
         
-        # WARNING: ntaxa and nchar in format string does not give us the right answer!
+        # WARNING: what if ntaxa and nchar in format string does not give us the right answer?
         # Should we raise an error instead?
         if self.ntaxa != len(self.taxa):
             self.ntaxa = len(self.taxa)
@@ -483,7 +496,7 @@ class DataHandler(GenericHandler):
                     continue 
                 else:
                     if key == 'symbols':
-                        value = '"%s"' % value
+                        value = '"%s"' % "".join(sorted(self.symbols))
                     fstring.append("%s=%s" % (key, value))
             return " ".join(fstring) + ";"
 
@@ -560,7 +573,7 @@ class NexusReader(object):
         :return: None
         """
         self.filename = "<String>"
-        self._read(io.StringIO(contents))
+        self._read(io.StringIO(unicode(contents)))
     
     def _read(self, handle):
         """Reads from a iterable object"""
