@@ -1,12 +1,13 @@
 """Tests for nexus reading"""
 import os
 import re
+import unittest
 from nexus import NexusReader
 from nexus.reader import GenericHandler, DataHandler, TreeHandler
 
-EXAMPLE_DIR = os.path.join(os.path.split(os.path.dirname(__file__))[0], 'examples')
+EXAMPLE_DIR = os.path.join(os.path.split(os.path.dirname(__file__))[0], '../examples')
 
-class Test_NexusReader_Core:
+class Test_NexusReader_Core(unittest.TestCase):
     """Test the Core functionality of NexusReader"""
     def test_read_file(self):
         nex = NexusReader(os.path.join(EXAMPLE_DIR, 'example.nex'))
@@ -46,7 +47,7 @@ class Test_NexusReader_Core:
         assert text == nex.write()
 
 
-class Test_DataHandler_SimpleNexusFormat:
+class Test_DataHandler_SimpleNexusFormat(unittest.TestCase):
     expected = {
         'Harry': ['0', '0'],
         'Simon': ['0', '1'],
@@ -130,7 +131,7 @@ class Test_DataHandler_SimpleNexusFormat:
         ]
         written = self.nex.write()
         for expected in expected_patterns:
-            assert re.search(expected, written, re.MULTILINE)
+            assert re.search(expected, written, re.MULTILINE), 'Expected "%s"' % expected
 
     def test__load_characters(self):
         for site, data in self.nex.data.characters.items():
@@ -163,11 +164,9 @@ class Test_DataHandler_SimpleNexusFormat:
             Harry              11111
             ;""")
         assert nex.data.nchar == 5
+        
 
-
-            
-
-class Test_DataHandler_InterleavedNexusFormat:
+class Test_DataHandler_InterleavedNexusFormat(unittest.TestCase):
     def test_interleave_matrix_parsing(self):
         nexus = NexusReader(os.path.join(EXAMPLE_DIR, 'example3.nex'))
         assert nexus.data.ntaxa == 2 == len(nexus.data.taxa)
@@ -178,7 +177,7 @@ class Test_DataHandler_InterleavedNexusFormat:
     
     
 
-class Test_DataHandler_AlternateNexusFormat:
+class Test_DataHandler_AlternateNexusFormat(unittest.TestCase):
     def setUp(self):
         self.nex = NexusReader(os.path.join(EXAMPLE_DIR, 'example2.nex'))
         
@@ -189,7 +188,7 @@ class Test_DataHandler_AlternateNexusFormat:
         # the alternate format has a distinct taxa and characters block, and
         # we need to estimate the number of taxa in a different way.
         assert self.nex.data.ntaxa == 4
-        
+    
     def test_format_string(self):
         expected = {
             'datatype': 'dna',
@@ -225,7 +224,7 @@ class Test_DataHandler_AlternateNexusFormat:
             assert self.nex.data.matrix[k] == ['a', 'c', 't', 'g']
 
 
-class Test_DataHandler_CharacterBlockNexusFormat:
+class Test_DataHandler_CharacterBlockNexusFormat(unittest.TestCase):
     def setUp(self):
         self.nex = NexusReader(os.path.join(EXAMPLE_DIR, 'example-characters.nex'))
         
@@ -267,7 +266,7 @@ class Test_DataHandler_CharacterBlockNexusFormat:
                 assert self.nex.data.characters["CHAR_%s" % site][taxon] == site
 
 
-class Test_TaxaHandler_AlternateNexusFormat:
+class Test_TaxaHandler_AlternateNexusFormat(unittest.TestCase):
     expected = ['John', 'Paul', 'George', 'Ringo']
     def setUp(self):
         self.nex = NexusReader(os.path.join(EXAMPLE_DIR, 'example2.nex'))
@@ -286,7 +285,7 @@ class Test_TaxaHandler_AlternateNexusFormat:
             assert taxa == self.expected[idx]
     
 
-class Test_TreeHandler_SimpleTreefile:
+class Test_TreeHandler_SimpleTreefile(unittest.TestCase):
     def setUp(self):
         self.nex = NexusReader(os.path.join(EXAMPLE_DIR, 'example.trees'))
     
@@ -299,6 +298,12 @@ class Test_TreeHandler_SimpleTreefile:
         assert len(self.nex.blocks['trees'].trees) == 3 == self.nex.blocks['trees'].ntrees
         assert len(self.nex.trees.trees) == 3 == self.nex.trees.ntrees
         
+    def test_taxa(self):
+        expected = ['Chris', 'Bruce', 'Tom', 'Henry', 'Timothy', 'Mark', 'Simon', 'Fred', 'Kevin', 'Roger', 'Michael', 'Andrew', 'David']
+        assert len(self.nex.trees.taxa) == len(expected)
+        for taxon in expected:
+            assert taxon in self.nex.trees.taxa
+        
     def test_iterable(self):
         for tree in self.nex.blocks['trees']:
             pass
@@ -313,7 +318,7 @@ class Test_TreeHandler_SimpleTreefile:
         assert expected == written
     
 
-class Test_TreeHandler_TranslatedTreefile:
+class Test_TreeHandler_TranslatedTreefile(unittest.TestCase):
     def setUp(self):
         self.nex = NexusReader(os.path.join(EXAMPLE_DIR, 'example-translated.trees'))
     
@@ -332,6 +337,12 @@ class Test_TreeHandler_TranslatedTreefile:
         for tree in self.nex.trees:
             tree
     
+    def test_taxa(self):
+        expected = ['Chris', 'Bruce', 'Tom', 'Henry', 'Timothy', 'Mark', 'Simon', 'Fred', 'Kevin', 'Roger', 'Michael', 'Andrew', 'David']
+        assert len(self.nex.trees.taxa) == len(expected)
+        for taxon in expected:
+            assert taxon in self.nex.trees.taxa
+    
     def test_was_translated_flag_set(self):
         assert self.nex.trees.was_translated == True
         
@@ -368,7 +379,7 @@ class Test_TreeHandler_TranslatedTreefile:
         assert other_tree_file.trees[0] == self.nex.trees[0]
     
 
-class Test_TreeHandler_BEAST_Format:
+class Test_TreeHandler_BEAST_Format(unittest.TestCase):
 
     def setUp(self):
         self.nex = NexusReader(os.path.join(EXAMPLE_DIR, 'example_beast.trees'))
@@ -380,6 +391,17 @@ class Test_TreeHandler_BEAST_Format:
         # did we get a tree block?
         assert 'trees' in self.nex.blocks
 
+    def test_taxa(self):
+        expected = [
+            "R1", "B2", "S3", "T4", "A5", "E6", "U7", "T8", "T9", "F10", "U11", 
+            "T12", "N13", "F14", "K15", "N16", "I17", "L18", "S19", "T20", "V21", 
+            "R22", "M23", "H24", "M25", "M26", "M27", "R28", "T29", "M30", "P31", 
+            "T32", "R33", "P34", "R35", "W36", "F37", "F38"
+        ]
+        assert len(self.nex.trees.taxa) == len(expected)
+        for taxon in expected:
+            assert taxon in self.nex.trees.taxa
+            
     def test_treecount(self):
         assert len(self.nex.blocks['trees'].trees) == 1 == self.nex.blocks['trees'].ntrees
         assert len(self.nex.trees.trees) == 1 == self.nex.trees.ntrees
@@ -390,17 +412,15 @@ class Test_TreeHandler_BEAST_Format:
     def test_parsing_sets_translators(self):
         assert len(self.nex.trees.translators) == 38 # 38 taxa in example trees
 
-    # def test_detranslate_BEAST_format_extended(self):
-    #     print self.nex.trees[0]
-    #     self.nex.trees.detranslate()
-    #     print self.nex.trees[0]
-    #     for index, taxon in self.nex.trees.translators.items():
-    #         # check if the taxon name is present in the tree...
-    #         assert taxon in self.nex.trees[0], "Expecting taxon %s in tree description" % taxon
-    #     assert self.nex.trees._been_detranslated == True
+    def test_detranslate_BEAST_format_extended(self):
+        self.nex.trees.detranslate()
+        for index, taxon in self.nex.trees.translators.items():
+            # check if the taxon name is present in the tree...
+            assert taxon in self.nex.trees[0], "Expecting taxon %s in tree description" % taxon
+        assert self.nex.trees._been_detranslated == True
     
 
-class Test_TreeHandler__detranslate_tree:
+class Test_TreeHandler__detranslate_tree(unittest.TestCase):
     
     def test_no_change(self):
         translatetable = {'0': 'Chris', '1': 'Bruce', '2': 'Tom'}
@@ -438,57 +458,87 @@ class Test_TreeHandler__detranslate_tree:
         assert trans == newtree, "Unable to correctly detranslate a BEAST tree"
 
 
-
-def test_regression_whitespace_in_matrix():
+class test_Regression_whitespace_in_matrix(unittest.TestCase):
     """Regression: Test that leading whitespace in a data matrix is parsed ok"""
-    nex = NexusReader()
-    nex.read_string("""
-    #NEXUS 
+    def test_regression(self):
+        nex = NexusReader()
+        nex.read_string("""
+        #NEXUS 
     
-    Begin data;
-        Dimensions ntax=4 nchar=2;
-                            Format datatype=standard symbols="01" gap=-;
-            Matrix
-    Harry              00
-            Simon              01
-                    Betty              10
-                                Louise 11
-        ;
-    End;
-    """)
-    assert nex.blocks['data'].nchar == 2
-    assert nex.blocks['data'].matrix['Harry'] == ['0', '0']
-    assert nex.blocks['data'].matrix['Simon'] == ['0', '1']
-    assert nex.blocks['data'].matrix['Betty'] == ['1', '0']
-    assert nex.blocks['data'].matrix['Louise'] == ['1', '1']
+        Begin data;
+            Dimensions ntax=4 nchar=2;
+                                Format datatype=standard symbols="01" gap=-;
+                Matrix
+        Harry              00
+                Simon              01
+                        Betty              10
+                                    Louise 11
+            ;
+        End;
+        """)
+        assert nex.blocks['data'].nchar == 2
+        assert nex.blocks['data'].matrix['Harry'] == ['0', '0']
+        assert nex.blocks['data'].matrix['Simon'] == ['0', '1']
+        assert nex.blocks['data'].matrix['Betty'] == ['1', '0']
+        assert nex.blocks['data'].matrix['Louise'] == ['1', '1']
         
     
 
-def test_badchars_quoted_in_taxaname():
-    nex = NexusReader()
-    nex.read_string("""
-    #NEXUS
+class test_badchars_quoted_in_taxaname(unittest.TestCase):
+    def test_regressions(self):
+        nex = NexusReader()
+        nex.read_string("""
+        #NEXUS
 
-    Begin trees;
-    	Translate
-    		1 MANGIC_Bugan,
-    		2 MANGIC_Paliu,
-    		3 MANGIC_Mang,
-    		4 PALAUNGWA_Danaw,
-    		5 'PALAUNGWA_De.Ang'
-        ;
-        tree 1 = (1,2,3,4,5);
-    """)
-    # did we get a tree block?
-    assert 'trees' in nex.blocks
-    # did we find 3 trees?
-    assert len(nex.blocks['trees'].trees) == 1 == nex.blocks['trees'].ntrees
-    # did we get the translation parsed properly.
-    assert nex.trees.was_translated == True
-    assert len(nex.trees.translators) == 5 # 5 taxa in example trees
-    # check last entry
-    assert nex.trees.translators['5'] == 'PALAUNGWA_De.Ang'
-    # check detranslate
-    nex.trees.detranslate()
-    assert '(MANGIC_Bugan,MANGIC_Paliu,MANGIC_Mang,PALAUNGWA_Danaw,PALAUNGWA_De.Ang)' in nex.trees[0]
-    
\ No newline at end of file
+        Begin trees;
+        	Translate
+        		1 MANGIC_Bugan,
+        		2 MANGIC_Paliu,
+        		3 MANGIC_Mang,
+        		4 PALAUNGWA_Danaw,
+        		5 'PALAUNGWA_De.Ang'
+            ;
+            tree 1 = (1,2,3,4,5);
+        """)
+        # did we get a tree block?
+        assert 'trees' in nex.blocks
+        # did we find 3 trees?
+        assert len(nex.blocks['trees'].trees) == 1 == nex.blocks['trees'].ntrees
+        # did we get the translation parsed properly.
+        assert nex.trees.was_translated == True
+        assert len(nex.trees.translators) == 5 # 5 taxa in example trees
+        # check last entry
+        assert nex.trees.translators['5'] == 'PALAUNGWA_De.Ang'
+        # check detranslate
+        nex.trees.detranslate()
+        assert '(MANGIC_Bugan,MANGIC_Paliu,MANGIC_Mang,PALAUNGWA_Danaw,PALAUNGWA_De.Ang)' in nex.trees[0]
+    
+
+class Test_TreeHandler_Regression_RandomAPETrees(unittest.TestCase):
+    def test_regression(self):
+        nex = NexusReader()
+        nex.read_string(
+        """
+        #NEXUS
+        [R-package APE, Mon Apr  4 13:30:05 2011]
+
+        BEGIN TAXA;
+        	DIMENSIONS NTAX = 5;
+        	TAXLABELS
+        		t5
+        		t2
+        		t3
+        		t4
+        		t1
+        	;
+        END;
+        BEGIN TREES;
+        	TREE * UNTITLED = [&R] (((t5:0.8158685302,(t2:0.3804786047,t3:0.9345045802):0.9044287337):0.2170910214,t4:0.5744336853):0.9122619091,t1:0.2579922327);
+        	TREE * UNTITLED = [&R] (((t1:0.7530630897,t5:0.00636632205):0.8808043781,t2:0.967890667):0.861689928,(t3:0.795280267,t4:0.4398460181):0.2651306945);
+        END;
+        """
+        )
+        assert nex.trees.ntrees == 2
+
+if __name__ == '__main__':
+    unittest.main()
